package geniusweb.events;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonTypeName;

import geniusweb.actions.Action;

/**
 * Event where a party did an {@link Action}
 *
 */
@JsonTypeName("actionevent")
public class ActionEvent extends AbstractEvent {

	private final Action action;

	/**
	 * 
	 * @param action
	 * @param time   the current time to use. see
	 *               {@link System#currentTimeMillis()}.
	 */
	@JsonCreator
	public ActionEvent(@JsonProperty("action") Action action,
			@JsonProperty("time") Long time) {
		super(time);
		if (action == null) {
			throw new IllegalArgumentException("Action must not be null");
		}
		this.action = action;
	}

	/**
	 * 
	 * @return action done by some party in the system.
	 */
	public Action getAction() {
		return action;
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName() + "[" + getTime() + "," + action
				+ "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + ((action == null) ? 0 : action.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		ActionEvent other = (ActionEvent) obj;
		if (action == null) {
			if (other.action != null)
				return false;
		} else if (!action.equals(other.action))
			return false;
		return true;
	}

}
