package events;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.mock;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.actions.Action;
import geniusweb.actions.EndNegotiation;
import geniusweb.actions.PartyId;
import geniusweb.events.ActionEvent;
import geniusweb.events.NegotiationEvent;
import tudelft.utilities.junit.GeneralTests;

public class ActionEventTest extends GeneralTests<ActionEvent> {
	private static final long NOW = 101l;
	private final ObjectMapper jackson = new ObjectMapper();
	private final String string = "{\"actionevent\":{\"action\":{\"endnegotiation\":{\"actor\":\"party1\"}},\"time\":101}}";
	private final PartyId id = new PartyId("party1");
	private final Action action = new EndNegotiation(id);
	private final ActionEvent evt = new ActionEvent(action, NOW);
	private final ActionEvent evt1 = new ActionEvent(action, NOW);
	private final ActionEvent evtb = new ActionEvent(action, NOW + 1);

	@Override
	public List<List<ActionEvent>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(evt, evt1), Arrays.asList(evtb));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("ActionEvent\\[.*" + NOW + ".*EndNegotiation.*\\]",
				"ActionEvent\\[.*" + (NOW + 1) + ".*EndNegotiation.*\\]");
	}

	@Test
	public void smoketest() {
		Action action = mock(EndNegotiation.class);
		ActionEvent evt = new ActionEvent(action, 0l);
	}

	@Test
	public void serializeTest() throws JsonProcessingException {
		System.out.println(jackson.writeValueAsString(evt));
		assertEquals(string, jackson.writeValueAsString(evt));
	}

	@Test
	public void deserializeTestReadActionEvent() throws IOException {
		ActionEvent evt1 = jackson.readValue(string, ActionEvent.class);
		// compare fields, as evt is a derived/new inner class
		assertEquals(EndNegotiation.class, evt1.getAction().getClass());
		assertEquals((Long) 101l, evt1.getTime());
	}

	@Test
	public void deserializeTestReadEvent() throws IOException {
		NegotiationEvent evt1 = jackson.readValue(string,
				NegotiationEvent.class);
		assertEquals(ActionEvent.class, evt1.getClass());
		ActionEvent ev1 = (ActionEvent) evt1;
		assertEquals(EndNegotiation.class, ev1.getAction().getClass());
		assertEquals((Long) 101l, ev1.getTime());
	}

}
